@@ -213,7 +213,7 @@ The pound sign # denotes a comment.
 @subsection FRIENDLYNAME
 The FRIENDLYNAME field is an arbitrary text used to identify the reader.
 This text is displayed by commands like @c pcsc_scan
-http://ludovic.rousseau.free.fr/softwares/pcsc-tools/ that prints the
+https://pcsc-tools.apdu.fr/ that prints the
 names of all the connected and detected readers.
 
 @subsection DEVICENAME
@@ -315,7 +315,7 @@ whichever location your reader resides.
 	SCARD_IO_HEADER, *PSCARD_IO_HEADER;
 
 	/*
-	 * The list of tags should be alot more but this is all I use in the
+	 * The list of tags should be a lot more but this is all I use in the
 	 * meantime
 	 */
 #define TAG_IFD_ATR                     0x0303	/**< ATR */
@@ -459,7 +459,7 @@ port used by each reader.
   @endcode
 
   bInterfaceNumber is the number of the interface on the device. It is
-  only usefull for devices with more than one CCID interface.
+  only useful for devices with more than one CCID interface.
 
   devpath is the filename of the device on the file system.
 
