@@ -145,6 +145,10 @@ typedef struct
 	 */
 	int IFD_bcdDevice;
 
+#ifdef USE_COMPOSITE_AS_MULTISLOT
+	int num_interfaces;
+#endif
+
 	/*
 	 * Gemalto extra features, if any
 	 */
@@ -257,7 +261,10 @@ typedef struct
 #define ACS_ACR1252				0x072F223B
 #define ACS_ACR1252IMP			0x072F2259
 #define ACS_ACR1552				0x072F2303
+#define KAPELSE_KAPLIN2         	0x29470105
+#define KAPELSE_KAPECV          	0x29470112
 
+#define VENDOR_KAPELSE 0x2947
 #define VENDOR_GEMALTO 0x08E6
 #define GET_VENDOR(readerID) ((readerID >> 16) & 0xFFFF)
 
