@@ -114,7 +114,7 @@ typedef struct
 	/*
 	 * does the reader echoes the serial communication bytes?
 	 */
-	int echo;
+	bool echo;
 
 	/*
 	 * serial communication buffer
@@ -339,7 +339,7 @@ ack:
 
 	if (echo)
 	{
-		echo = FALSE;
+		echo = false;
 		goto start;
 	}
 
@@ -520,7 +520,7 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 {
 	int readerID;
 	int i;
-	int already_used = FALSE;
+	bool already_used = false;
 	static int previous_reader_index = -1;
 
 	readerID = GEMPCTWIN;
@@ -541,7 +541,7 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 		if (serialDevice[i].device
 			&& strcmp(serialDevice[i].device, dev_name) == 0)
 		{
-			already_used = TRUE;
+			already_used = true;
 
 			DEBUG_COMM2("%s already used. Multi-slot reader?", dev_name);
 			break;
@@ -631,9 +631,9 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 	serialDevice[reader_index].ccid.gemalto_firmware_features = NULL;
 	serialDevice[reader_index].ccid.dwProtocols = SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1;
 #ifdef ENABLE_ZLP
-	serialDevice[reader_index].ccid.zlp = FALSE;
+	serialDevice[reader_index].ccid.zlp = false;
 #endif
-	serialDevice[reader_index].echo = TRUE;
+	serialDevice[reader_index].echo = true;
 
 	/* change some values depending on the reader */
 	switch (readerID)
@@ -641,14 +641,14 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 		case GEMCOREPOSPRO:
 			serialDevice[reader_index].ccid.bMaxSlotIndex = 4;	/* 5 slots */
 			serialDevice[reader_index].ccid.arrayOfSupportedDataRates = SerialExtendedDataRates;
-			serialDevice[reader_index].echo = FALSE;
+			serialDevice[reader_index].echo = false;
 			serialDevice[reader_index].ccid.dwMaxDataRate = 500000;
 			break;
 
 		case GEMCORESIMPRO:
 			serialDevice[reader_index].ccid.bMaxSlotIndex = 1; /* 2 slots */
 			serialDevice[reader_index].ccid.arrayOfSupportedDataRates = SerialExtendedDataRates;
-			serialDevice[reader_index].echo = FALSE;
+			serialDevice[reader_index].echo = false;
 			serialDevice[reader_index].ccid.dwMaxDataRate = 500000;
 			break;
 
@@ -656,7 +656,7 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 			serialDevice[reader_index].ccid.dwDefaultClock = 4800;
 			serialDevice[reader_index].ccid.bMaxSlotIndex = 1; /* 2 slots */
 			serialDevice[reader_index].ccid.arrayOfSupportedDataRates = SIMPro2DataRates;
-			serialDevice[reader_index].echo = FALSE;
+			serialDevice[reader_index].echo = false;
 			serialDevice[reader_index].ccid.dwMaxDataRate = 825806;
 			break;
 
@@ -672,7 +672,7 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 			serialDevice[reader_index].ccid.dwMaxDataRate = 826000;
 			serialDevice[reader_index].ccid.arrayOfSupportedDataRates = NULL;
 			serialDevice[reader_index].ccid.bMaxSlotIndex = 1;	/* 2 slots */
-			serialDevice[reader_index].echo = FALSE;
+			serialDevice[reader_index].echo = false;
 			break;
 
 	}
@@ -880,7 +880,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 
 	/* perform a command to configure GemPC Twin reader card movement
 	 * notification to synchronous mode: the card movement is notified _after_
-	 * the host command and _before_ the reader anwser */
+	 * the host command and _before_ the reader answer */
 	if (0 != strcasecmp(reader_name,"SEC1210"))
 	{
 		unsigned char tx_buffer[] = { 0x01, 0x01, 0x01};
@@ -922,7 +922,7 @@ status_t CloseSerial(unsigned int reader_index)
 	/* Decrement number of opened slot */
 	(*serialDevice[reader_index].nb_opened_slots)--;
 
-	/* release the allocated ressources for the last slot only */
+	/* release the allocated resources for the last slot only */
 	if (0 == *serialDevice[reader_index].nb_opened_slots)
 	{
 		DEBUG_COMM("Last slot closed. Release resources");
